@@ -17,6 +17,8 @@ from oslo_utils import timeutils
 from sqlalchemy import asc
 from sqlalchemy.orm import exc
 
+from neutron_lib.db import api as db_api
+
 from networking_l2gw.db.l2gateway.ovsdb import models
 
 LOG = logging.getLogger(__name__)
@@ -25,7 +27,7 @@ LOG = logging.getLogger(__name__)
 def add_vlan_binding(context, record_dict):
     """Insert a vlan binding of a given physical port."""
     session = context.session
-    with session.begin(subtransactions=True):
+    with db_api.CONTEXT_WRITER.using(context):
         binding = models.VlanBindings(
             port_uuid=record_dict['port_uuid'],
             vlan=record_dict['vlan'],
@@ -37,8 +39,9 @@ def add_vlan_binding(context, record_dict):
 def delete_vlan_binding(context, record_dict):
     """Delete vlan bindings of a given physical port."""
     session = context.session
-    with session.begin(subtransactions=True):
-        if(record_dict['vlan'] and record_dict['logical_switch_uuid']):
+    with db_api.CONTEXT_WRITER.using(context):
+        if (record_dict['vlan'] is not None and
+                record_dict['logical_switch_uuid']):
             session.query(models.VlanBindings).filter_by(
                 port_uuid=record_dict['port_uuid'], vlan=record_dict['vlan'],
                 logical_switch_uuid=record_dict['logical_switch_uuid'],
@@ -48,7 +51,7 @@ def delete_vlan_binding(context, record_dict):
 def add_physical_locator(context, record_dict):
     """Insert a new physical locator."""
     session = context.session
-    with session.begin(subtransactions=True):
+    with db_api.CONTEXT_WRITER.using(context):
         locator = models.PhysicalLocators(
             uuid=record_dict['uuid'],
             dst_ip=record_dict['dst_ip'],
@@ -59,7 +62,7 @@ def add_physical_locator(context, record_dict):
 def delete_physical_locator(context, record_dict):
     """Delete physical locator that matches the supplied uuid."""
     session = context.session
-    with session.begin(subtransactions=True):
+    with db_api.CONTEXT_WRITER.using(context):
         if(record_dict['uuid']):
             session.query(models.PhysicalLocators).filter_by(
                 uuid=record_dict['uuid'],
@@ -69,7 +72,7 @@ def delete_physical_locator(context, record_dict):
 def add_physical_switch(context, record_dict):
     """Insert a new physical switch."""
     session = context.session
-    with session.begin(subtransactions=True):
+    with db_api.CONTEXT_WRITER.using(context):
         physical_switch = models.PhysicalSwitches(
             uuid=record_dict['uuid'],
             name=record_dict['name'],
@@ -82,7 +85,7 @@ def add_physical_switch(context, record_dict):
 def delete_physical_switch(context, record_dict):
     """Delete physical switch that matches the supplied uuid."""
     session = context.session
-    with session.begin(subtransactions=True):
+    with db_api.CONTEXT_WRITER.using(context):
         if(record_dict['uuid']):
             session.query(models.PhysicalSwitches).filter_by(
                 uuid=record_dict['uuid'],
@@ -92,7 +95,7 @@ def delete_physical_switch(context, record_dict):
 def add_logical_switch(context, record_dict):
     """Insert a new logical switch."""
     session = context.session
-    with session.begin(subtransactions=True):
+    with db_api.CONTEXT_WRITER.using(context):
         logical_switch = models.LogicalSwitches(
             uuid=record_dict['uuid'],
             name=record_dict['name'],
@@ -104,7 +107,7 @@ def add_logical_switch(context, record_dict):
 def delete_logical_switch(context, record_dict):
     """delete logical switch that matches the supplied uuid."""
     session = context.session
-    with session.begin(subtransactions=True):
+    with db_api.CONTEXT_WRITER.using(context):
         if(record_dict['uuid']):
             session.query(models.LogicalSwitches).filter_by(
                 uuid=record_dict['uuid'],
@@ -114,7 +117,7 @@ def delete_logical_switch(context, record_dict):
 def add_physical_port(context, record_dict):
     """Insert a new physical port."""
     session = context.session
-    with session.begin(subtransactions=True):
+    with db_api.CONTEXT_WRITER.using(context):
         physical_port = models.PhysicalPorts(
             uuid=record_dict['uuid'],
             name=record_dict['name'],
@@ -126,7 +129,7 @@ def add_physical_port(context, record_dict):
 
 def update_physical_ports_status(context, record_dict):
     """Update physical port fault status."""
-    with context.session.begin(subtransactions=True):
+    with db_api.CONTEXT_WRITER.using(context):
         (context.session.query(models.PhysicalPorts).
          filter(models.PhysicalPorts.uuid == record_dict['uuid']).
          update({'port_fault_status': record_dict['port_fault_status']},
@@ -145,7 +148,7 @@ def update_physical_switch_status(context, record_dict):
 def delete_physical_port(context, record_dict):
     """Delete physical port that matches the supplied uuid."""
     session = context.session
-    with session.begin(subtransactions=True):
+    with db_api.CONTEXT_WRITER.using(context):
         if(record_dict['uuid']):
             session.query(models.PhysicalPorts).filter_by(
                 uuid=record_dict['uuid'],
@@ -155,7 +158,7 @@ def delete_physical_port(context, record_dict):
 def add_ucast_mac_local(context, record_dict):
     """Insert a new ucast mac local."""
     session = context.session
-    with session.begin(subtransactions=True):
+    with db_api.CONTEXT_WRITER.using(context):
         ucast_mac_local = models.UcastMacsLocals(
             uuid=record_dict['uuid'],
             mac=record_dict['mac'],
@@ -169,7 +172,7 @@ def add_ucast_mac_local(context, record_dict):
 def delete_ucast_mac_local(context, record_dict):
     """Delete ucast mac local that matches the supplied uuid."""
     session = context.session
-    with session.begin(subtransactions=True):
+    with db_api.CONTEXT_WRITER.using(context):
         if(record_dict['uuid']):
             session.query(models.UcastMacsLocals).filter_by(
                 uuid=record_dict['uuid'],
@@ -179,7 +182,7 @@ def delete_ucast_mac_local(context, record_dict):
 def add_ucast_mac_remote(context, record_dict):
     """Insert a new ucast mac remote."""
     session = context.session
-    with session.begin(subtransactions=True):
+    with db_api.CONTEXT_WRITER.using(context):
         ucast_mac_remote = models.UcastMacsRemotes(
             uuid=record_dict['uuid'],
             mac=record_dict['mac'],
@@ -193,7 +196,7 @@ def add_ucast_mac_remote(context, record_dict):
 def update_ucast_mac_remote(context, rec_dict):
     """Update ucast mac remote."""
     try:
-        with context.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(context):
             (context.session.query(models.UcastMacsRemotes).filter_by(
                 uuid=rec_dict['uuid'],
                 ovsdb_identifier=rec_dict['ovsdb_identifier']).update(
@@ -209,7 +212,7 @@ def update_ucast_mac_remote(context, rec_dict):
 def delete_ucast_mac_remote(context, record_dict):
     """Delete ucast mac remote that matches the supplied uuid."""
     session = context.session
-    with session.begin(subtransactions=True):
+    with db_api.CONTEXT_WRITER.using(context):
         if(record_dict['uuid']):
             session.query(models.UcastMacsRemotes).filter_by(
                 uuid=record_dict['uuid'],
@@ -270,10 +273,11 @@ def get_ucast_mac_remote(context, record_dict):
 def get_ucast_mac_local(context, record_dict):
     """Get ucast macs local that matches the uuid and ovsdb_identifier."""
     try:
-        query = context.session.query(models.UcastMacsLocals)
-        local_mac = query.filter_by(
-            uuid=record_dict['uuid'],
-            ovsdb_identifier=record_dict['ovsdb_identifier']).one()
+        with db_api.CONTEXT_READER.using(context):
+            query = context.session.query(models.UcastMacsLocals)
+            local_mac = query.filter_by(
+                uuid=record_dict['uuid'],
+                ovsdb_identifier=record_dict['ovsdb_identifier']).one()
     except exc.NoResultFound:
         LOG.debug('no Local mac found for %s and %s',
                   record_dict['uuid'],
@@ -349,10 +353,11 @@ def get_physical_locator_by_dst_ip(context, record_dict):
 def get_logical_switch_by_name(context, record_dict):
     """Get logical switch that matches the supplied name."""
     try:
-        query = context.session.query(models.LogicalSwitches)
-        logical_switch = query.filter_by(
-            name=record_dict['logical_switch_name'],
-            ovsdb_identifier=record_dict['ovsdb_identifier']).one()
+        with db_api.CONTEXT_READER.using(context):
+            query = context.session.query(models.LogicalSwitches)
+            logical_switch = query.filter_by(
+                name=record_dict['logical_switch_name'],
+                ovsdb_identifier=record_dict['ovsdb_identifier']).one()
     except exc.NoResultFound:
         LOG.debug('no logical switch found for %s and %s',
                   record_dict['logical_switch_name'],
@@ -437,7 +442,7 @@ def add_pending_ucast_mac_remote(context, operation,
                                  mac_remotes):
     """Insert a pending ucast_mac_remote (insert/update/delete)."""
     session = context.session
-    with session.begin(subtransactions=True):
+    with db_api.CONTEXT_WRITER.using(context):
         for mac in mac_remotes:
             pending_mac = models.PendingUcastMacsRemote(
                 uuid=mac.get('uuid', None),
@@ -460,9 +465,9 @@ def delete_pending_ucast_mac_remote(context, operation,
                                     mac_remote):
     """Delete a pending ucast_mac_remote."""
     session = context.session
-    with session.begin(subtransactions=True):
-        if(mac_remote and logical_switch_id
-           and ovsdb_identifier and operation):
+    with db_api.CONTEXT_WRITER.using(context):
+        if(mac_remote and logical_switch_id and
+           ovsdb_identifier and operation):
             query = session.query(models.PendingUcastMacsRemote).filter_by(
                 mac=mac_remote,
                 ovsdb_identifier=ovsdb_identifier,
@@ -489,8 +494,8 @@ def get_pending_ucast_mac_remote(context, ovsdb_identifier, mac,
 
 def get_all_pending_remote_macs_in_asc_order(context, ovsdb_identifier):
     """Get all the pending remote macs in ascending order of timestamp."""
-    session = context.session
-    with session.begin():
+    with db_api.CONTEXT_READER.using(context):
+        session = context.session
         return session.query(
             models.PendingUcastMacsRemote
             ).filter_by(ovsdb_identifier=ovsdb_identifier
@@ -500,8 +505,8 @@ def get_all_pending_remote_macs_in_asc_order(context, ovsdb_identifier):
 
 def get_all_ucast_mac_remote_by_ls(context, record_dict):
     """Get ucast macs remote that matches ls_id and ovsdb_identifier."""
-    session = context.session
-    with session.begin():
+    with db_api.CONTEXT_READER.using(context):
+        session = context.session
         return session.query(models.UcastMacsRemotes).filter_by(
             ovsdb_identifier=record_dict['ovsdb_identifier'],
             logical_switch_id=record_dict['logical_switch_id']).all()
@@ -511,7 +516,7 @@ def delete_all_physical_locators_by_ovsdb_identifier(context,
                                                      ovsdb_identifier):
     """Delete all physical locators based on ovsdb identifier."""
     session = context.session
-    with session.begin(subtransactions=True):
+    with db_api.CONTEXT_WRITER.using(context):
         session.query(models.PhysicalLocators).filter_by(
             ovsdb_identifier=ovsdb_identifier).delete()
 
@@ -520,7 +525,7 @@ def delete_all_physical_switches_by_ovsdb_identifier(context,
                                                      ovsdb_identifier):
     """Delete all physical switches based on ovsdb identifier."""
     session = context.session
-    with session.begin(subtransactions=True):
+    with db_api.CONTEXT_WRITER.using(context):
         session.query(models.PhysicalSwitches).filter_by(
             ovsdb_identifier=ovsdb_identifier).delete()
 
@@ -529,7 +534,7 @@ def delete_all_physical_ports_by_ovsdb_identifier(context,
                                                   ovsdb_identifier):
     """Delete all physical ports based on ovsdb identifier."""
     session = context.session
-    with session.begin(subtransactions=True):
+    with db_api.CONTEXT_WRITER.using(context):
         session.query(models.PhysicalPorts).filter_by(
             ovsdb_identifier=ovsdb_identifier).delete()
 
@@ -538,7 +543,7 @@ def delete_all_logical_switches_by_ovsdb_identifier(context,
                                                     ovsdb_identifier):
     """Delete all physical switches based on ovsdb identifier."""
     session = context.session
-    with session.begin(subtransactions=True):
+    with db_api.CONTEXT_WRITER.using(context):
         session.query(models.LogicalSwitches).filter_by(
             ovsdb_identifier=ovsdb_identifier).delete()
 
@@ -547,7 +552,7 @@ def delete_all_ucast_macs_locals_by_ovsdb_identifier(context,
                                                      ovsdb_identifier):
     """Delete all ucast mac locals based on ovsdb identifier."""
     session = context.session
-    with session.begin(subtransactions=True):
+    with db_api.CONTEXT_WRITER.using(context):
         session.query(models.UcastMacsLocals).filter_by(
             ovsdb_identifier=ovsdb_identifier).delete()
 
@@ -556,7 +561,7 @@ def delete_all_ucast_macs_remotes_by_ovsdb_identifier(context,
                                                       ovsdb_identifier):
     """Delete all ucast mac remotes based on ovsdb identifier."""
     session = context.session
-    with session.begin(subtransactions=True):
+    with db_api.CONTEXT_WRITER.using(context):
         session.query(models.UcastMacsRemotes).filter_by(
             ovsdb_identifier=ovsdb_identifier).delete()
 
@@ -565,6 +570,6 @@ def delete_all_vlan_bindings_by_ovsdb_identifier(context,
                                                  ovsdb_identifier):
     """Delete all vlan bindings based on ovsdb identifier."""
     session = context.session
-    with session.begin(subtransactions=True):
+    with db_api.CONTEXT_WRITER.using(context):
         session.query(models.VlanBindings).filter_by(
             ovsdb_identifier=ovsdb_identifier).delete()
